@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import ast
+import importlib
 import inspect
 import re
 import sys
@@ -150,8 +151,7 @@ def get_annotation_args(annotation: Any, module: str, class_name: str) -> tuple[
         return annotation.__parameters__  # type: ignore[no-any-return]
     result = getattr(annotation, "__args__", ())
     # 3.10 and earlier Tuple[()] returns ((), ) instead of () the tuple does
-    result = () if len(result) == 1 and result[0] == () else result  # type: ignore[misc]
-    return result
+    return () if len(result) == 1 and result[0] == () else result  # type: ignore[misc]
 
 
 def format_internal_tuple(t: tuple[Any, ...], config: Config) -> str:
@@ -244,7 +244,7 @@ def format_annotation(annotation: Any, config: Config) -> str:  # noqa: C901, PL
         fmt = [format_annotation(arg, config) for arg in args]
         formatted_args = f"\\[\\[{', '.join(fmt[:-1])}], {fmt[-1]}]"
     elif full_name == "typing.Literal":
-        formatted_args = "\\[{}]".format(", ".join(f"``{arg!r}``" for arg in args))
+        formatted_args = f"\\[{', '.join(f'``{arg!r}``' for arg in args)}]"
     elif full_name == "types.UnionType":
         return " | ".join([format_annotation(arg, config) for arg in args])
 
@@ -405,29 +405,56 @@ _TYPE_GUARD_IMPORTS_RESOLVED = set()
 _TYPE_GUARD_IMPORTS_RESOLVED_GLOBALS_ID = set()
 
 
-def _resolve_type_guarded_imports(autodoc_mock_imports: list[str], obj: Any) -> None:
-    if (hasattr(obj, "__module__") and obj.__module__ not in _TYPE_GUARD_IMPORTS_RESOLVED) or (
-        hasattr(obj, "__globals__") and id(obj.__globals__) not in _TYPE_GUARD_IMPORTS_RESOLVED_GLOBALS_ID
-    ):
-        _TYPE_GUARD_IMPORTS_RESOLVED.add(obj.__module__)
-        if obj.__module__ not in sys.builtin_module_names:
-            if hasattr(obj, "__globals__"):
-                _TYPE_GUARD_IMPORTS_RESOLVED_GLOBALS_ID.add(id(obj.__globals__))
+def _should_skip_guarded_import_resolution(obj: Any) -> bool:
+    if isinstance(obj, types.ModuleType):
+        return False  # Don't skip modules
 
-            module = inspect.getmodule(obj)
-            if module:
-                try:
-                    module_code = inspect.getsource(module)
-                except (TypeError, OSError):
-                    ...  # no source code => no type guards
-                else:
-                    for _, part in _TYPE_GUARD_IMPORT_RE.findall(module_code):
-                        guarded_code = textwrap.dedent(part)
-                        try:
-                            with mock(autodoc_mock_imports):
-                                exec(guarded_code, obj.__globals__)  # noqa: S102
-                        except Exception as exc:  # noqa: BLE001
-                            _LOGGER.warning(f"Failed guarded type import with {exc!r}")
+    if not hasattr(obj, "__globals__"):
+        return True  # Skip objects without __globals__
+
+    if hasattr(obj, "__module__"):
+        return obj.__module__ in _TYPE_GUARD_IMPORTS_RESOLVED or obj.__module__ in sys.builtin_module_names
+
+    return id(obj.__globals__) in _TYPE_GUARD_IMPORTS_RESOLVED_GLOBALS_ID
+
+
+def _execute_guarded_code(autodoc_mock_imports: list[str], obj: Any, module_code: str) -> None:
+    for _, part in _TYPE_GUARD_IMPORT_RE.findall(module_code):
+        guarded_code = textwrap.dedent(part)
+        try:
+            try:
+                with mock(autodoc_mock_imports):
+                    exec(guarded_code, getattr(obj, "__globals__", obj.__dict__))  # noqa: S102
+            except ImportError as exc:
+                # ImportError might have occurred because the module has guarded code as well,
+                # so we recurse on the module.
+                if exc.name:
+                    _resolve_type_guarded_imports(autodoc_mock_imports, importlib.import_module(exc.name))
+
+                    # Retry the guarded code and see if it works now after resolving all nested type guards.
+                    with mock(autodoc_mock_imports):
+                        exec(guarded_code, getattr(obj, "__globals__", obj.__dict__))  # noqa: S102
+        except Exception as exc:  # noqa: BLE001
+            _LOGGER.warning("Failed guarded type import with %r", exc)
+
+
+def _resolve_type_guarded_imports(autodoc_mock_imports: list[str], obj: Any) -> None:
+    if _should_skip_guarded_import_resolution(obj):
+        return
+
+    if hasattr(obj, "__globals__"):
+        _TYPE_GUARD_IMPORTS_RESOLVED_GLOBALS_ID.add(id(obj.__globals__))
+
+    module = inspect.getmodule(obj)
+
+    if module:
+        try:
+            module_code = inspect.getsource(module)
+        except (TypeError, OSError):
+            ...  # no source code => no type guards
+        else:
+            _TYPE_GUARD_IMPORTS_RESOLVED.add(module.__name__)
+            _execute_guarded_code(autodoc_mock_imports, obj, module_code)
 
 
 def _get_type_hint(autodoc_mock_imports: list[str], name: str, obj: Any) -> dict[str, Any]:
